/* 20 Jun 2002

   Interface from FORTRAN/GAMESS to C/system calls
   ===============================================
 
   The amount of quiche is kept to a minimum, as there are only 4 routines!
          The first three routines amount to a FORTRAN wrapper around
          the C calls malloc and free, by Steve Elbert in January 1990.
   memory allocation: function invocation only: LOCMEM = MEMGET(NWORDS)
          NWORDS is the no. of 8-byte words of memory requested.
          The return value is the address of the allocated memory, if
          this is zero, the system doesn't have the memory available.
   memory release: subroutine call only: CALL MEMREL(LOCMEM)
          Deallocate all memory above the address LOCMEM.
          (deallocated memory is not returned to system)
   memory address: function invocation only: LOCX = LADDRS(X)
          Returns the absolute address of X, in bytes.  This is needed
          only if the FORTRAN library does not include the LOC function.
   process sleeping: subroutine call only: CALL NAPTIME(seconds)
          which is used when MPI is being used to pass messages, and
          allows DDI data servers to sleep when they are not in use.

   Please note: 
   There really shouldn't be anything here for a machine other than these 4,
   so one should refrain from adding anything else to this file at all.
   Other routines are present only because of inadequate system libraries.
*/

/*--------------- Compaq (aka Digital) --------------*/

#ifdef COMPAQ
#include <stdlib.h>

long memget_(nwords) long *nwords;
   { size_t nbytes;
     nbytes = (size_t) (*nwords+2)*8;
     return (long) malloc(nbytes); }

#include <unistd.h>
void naptime_(nptime)
int *nptime;
{  unsigned int istat;
   istat=sleep(*nptime); }

#endif
 
/*----------------- Cray (all types) ----------------*/

#ifdef CRAY

int MEMGET(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void MEMREL(locmem) int *locmem;
   { free(*locmem); }

#include <unistd.h>
void NAPTIME(nptime)
int *nptime;
{  unsigned int istat;
   istat=sleep(*nptime); }

#endif

/*-------------- Fujitsu VPP and AP ---------------*/

#ifdef FUJITSU

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

int laddrs_(arg) int arg;
   { return(arg); }

/*   This is the only machine type missing a NAPTIME implementation   */
/*   If you are trying to link GAMESS, please send a copy of your     */
/*   version of NAPTIME (many examples here!) to Mike Schmidt         */

#endif

/*----------------- Hewlett-Packard -----------------*/
 
#ifdef HP9000

int memget(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) int *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime(nptime)
int *nptime;
{  unsigned int istat;
   istat=sleep((unsigned) *nptime); }

/*
      Special CPU timing routine for Hewlett-Packard
      Written by Shinichi Yamada,
      YHP (Yokogawa Hewlett-Packard), Tokyo, Japan.  in Nov 1992.
      wall timing modified by Dave Mullally (Convex) in Jun 1994.
      Francois Rotzinger (Lausanne) renamed etime --> hptme Jun 1995.
 
      This is a clone of the usual Unix ETIME function,
      except usage is as a subroutine call,
        REAL XTIME(2)
        CALL HPTME(XTIME)
 
      HPTME will return two values:
        xtime(1)      <==== user cpu time    ( sec )
        xtime(2)      <==== system cpu time  ( sec )
      add the latter two to get actual CPU time.
*/

#include <sys/types.h>
#include <sys/times.h>
#include <sys/time.h>
void hptme(hptme_val)
float *hptme_val;
{
struct tms      buf;
static float  clock_val=0.0;
if ( clock_val == 0.0 ) {
  clock_val = (float)sysconf(_SC_CLK_TCK);
}
times(&buf);
*hptme_val = (float)(buf.tms_utime);
*hptme_val = *hptme_val/clock_val;
hptme_val++;
*hptme_val = (float)(buf.tms_stime);
*hptme_val = *hptme_val/clock_val;
return;
}
void walltim(hptme1)
double  *hptme1;
{
struct timeval   time_sec;
struct timezone  tzone;
float  clock_val;
gettimeofday(&time_sec,&tzone);
*hptme1 = (double)time_sec.tv_sec;
return;
}

#endif

/*------------ International Business Machines ------------*/
/*  code is for 32 bit Power, Power2, and PowerPC systems, */
/*       or for 64 bit Power3 and Power4 systems.          */
/*  The term "system" can mean either workstations or SP.  */
 
#ifdef IBM32

int memget(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) int *locmem;
   { free(*locmem); }

int laddrs(arg) int arg;
   { return(arg); }

#include <unistd.h>
void naptime(time)
useconds_t *time;
{  int istat;
   istat=sleep(*time); }

#endif

#ifdef IBM64

long memget(nwords) long *nwords;
   { long nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) long *locmem;
   { free(*locmem); }

long laddrs(arg) int arg;
   { return(arg); }

#include <unistd.h>
void naptime(time)
long *time;
{  int istat;
   useconds_t delay;
   delay = (useconds_t) *time;
   istat=sleep(delay); }

#endif

/*----------------- PC running Linux -----------------*/

#ifdef gnu

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

int laddrs_(arg) int arg;
   { return(arg); }

#include <unistd.h>
void naptime_(nptime)
int *nptime;
{
  unsigned int istat;
  // printf("calling naptime ??? %d\n",*nptime);
  istat=sleep(*nptime); 
}

/*
     etime - return elapsed execution time
     double precision function etime (tarray)
     real tarray(2)
     written by Klaus-Peter Gulden, January 3, 1994
*/
#include <time.h>
#include <sys/times.h>

double xetime_(float *a)
{
  double        elapsed;
  struct tms    buf;
/*      CLK_TCK counts hundredths of seconds   */
  elapsed= (float) times(&buf)/ (float) CLK_TCK;
  a[0]=(float)(buf.tms_utime + buf.tms_cutime)/CLK_TCK;
  a[1]=(float)(buf.tms_stime + buf.tms_cstime)/CLK_TCK;
  return(elapsed);
}

/*
     fdate - return date and time in an ASCII string
     subroutine fdate(string)
     character*24 string

     returns the current date and time as a 24 character string in the
     format of ctime(3).  Neither `newline' nor NULL will be included.
     by Istvan Cserny, June 21, 1995
*/

/* void fdate_(char *cht, int cht_len) */
/* { */
/*     static time_t t; */
/*     time(&t); */
/*     strncpy(cht,ctime(&t),24L); */
/*     return; */
/* } */
/* needed by pgi */
/* void f__xargc(){ return 0; } */

#endif

/*--- Absoft compiler...one might find this in Linux or MAC OS X worlds ---*/

#ifdef ABSOFT

int MEMGET(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void MEMREL(locmem) int *locmem;
   { free(*locmem); }

int LADDRS(arg) int arg;
   { return(arg); }

#include <unistd.h>
void NAPTIME(nptime)
int *nptime;
{  unsigned int istat;
   istat=sleep(*nptime); }

#endif

/*----------------- Itanium (IA64) running Linux -----------------*/

#ifdef LINUXIA64

#include <stdlib.h>
#include <inttypes.h>
#include <malloc.h>

intptr_t memget_(nwords) intptr_t *nwords;
   { long long nbytes;
     nbytes = (*nwords+2)*8;
     return (intptr_t) malloc(nbytes); }

void memrel_(locmem) intptr_t *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime_(nptime)
int *nptime;
{  unsigned int istat;
   istat=sleep(*nptime); }

#endif
 
/*----------------- NEC SX series -----------------*/

#ifdef NECSX
#include <unistd.h>

void memget_(nwords,sxaddr)
 long long *nwords, **sxaddr ;
   {char *malloc(),*card;
    unsigned nbytes;
    nbytes = (unsigned) ((*nwords+2)*8);
    card =  malloc (nbytes);
    *sxaddr = (long long *) card; }

void memrel_(locmem)
 long long *locmem;
 {void free();
  char* ptrmem;
  ptrmem = (char *)locmem;
  free(ptrmem); }

void laddrs_(arg,addrarg)
long long *arg, **addrarg;
   { *addrarg = arg ; }

void naptime_(long *time)
  { long istat;
    istat=(long) sleep((unsigned)*time); }

#endif

/*-------------- Silicon Graphics, Incorporated --------------*/

#ifdef SGI32
#define FORTINT int
#endif
#ifdef SGI64
#define FORTINT long
#endif

#if (defined SGI32) || (defined SGI64)

int memget_(nwords) FORTINT *nwords;
   { FORTINT nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) FORTINT *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime_(FORTINT *time)
   { sginap((long) *time); }

#endif

/*----------------- SUN -----------------*/

#ifdef SUN

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime_(nptime)
useconds_t *nptime;
{  int istat;
   istat=sleep(*nptime); }

#endif
